@@ -1,3 +1,31 @@
+// [[Rcpp::depends(BH)]]
+/*!
+ * \brief       Implements a matrix with triangular storage layout
+ * 
+ * \author      O. Krause
+ * \date        2015
+ *
+ *
+ * \par Copyright 1995-2015 Shark Development Team
+ * 
+ * <BR><HR>
+ * This file is part of Shark.
+ * <http://image.diku.dk/shark/>
+ * 
+ * Shark is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published 
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * Shark is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with Shark.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ */
 #ifndef SHARK_LINALG_BLAS_TRIANGULAR_MATRIX_HPP
 #define SHARK_LINALG_BLAS_TRIANGULAR_MATRIX_HPP
 
@@ -30,7 +58,8 @@ public:
         typedef const matrix_reference<const self_type> const_closure_type;
         typedef matrix_reference<self_type> closure_type;
         typedef packed_tag storage_category;
-        typedef packed<Orientation,TriangularType> orientation;
+        typedef elementwise_tag evaluation_category;
+        typedef triangular<Orientation,TriangularType> orientation;
 
         // Construction and destruction
 
@@ -44,6 +73,7 @@ public:
 
         /** Packed matrix constructor with defined size and an initial value for all triangular matrix elements
          * \param size number of rows and columns
+         * \param init initial value of the non-zero elements
          */
         triangular_matrix(size_type size, scalar_type init):m_size(size),m_data(size * (size+1)/2,init) {}
 
@@ -59,7 +89,7 @@ public:
         triangular_matrix(matrix_expression<E> const& e)
                 :m_size(e().size1()), m_data(m_size * (m_size+1)/2)
         {
-                assign(e);
+                assign(*this, e);
         }
 
         // ---------
@@ -80,7 +110,7 @@ public:
         /// Grants low-level access to the matrix internals. Element order depends on whether the matrix is
         ///  row_major or column_major and upper or lower triangular
         const_pointer storage()const{
-                return &m_data[0];
+                return m_data.data();
         }
         
         ///\brief Returns the pointer to the beginning of the matrix storage
@@ -88,7 +118,7 @@ public:
         /// Grants low-level access to the matrix internals. Element order depends on whether the matrix is row_major or column_major.
         /// to access element (i,j) use storage()[i*stride1()+j*stride2()].
         pointer storage(){
-                return &m_data[0];
+                return m_data.data();
         }
         
         ///\brief Number of nonzero-elements stores in the matrix.
@@ -109,63 +139,40 @@ public:
                 m_size = size;
         }
         
+        void resize(size_type size1, size_type size2) {
+                SIZE_CHECK(size1 == size2);
+                resize(size1);
+                (void)size2;
+        }
+        
         void clear(){
                 std::fill(m_data.begin(), m_data.end(), value_type/*zero*/());
         }
 
         // Element access read only
         const_reference operator()(index_type i, index_type j) const {
+                SIZE_CHECK(i < size1());
+                SIZE_CHECK(j < size2());
                 if(!orientation::non_zero(i,j)) 
                         return value_type();
-                return m_data [orientation::element(i,j,size1())];
+                SIZE_CHECK(orientation::element(i,j,size1(),packed_tag())<m_data.size());
+                return m_data [orientation::element(i,j,size1(),packed_tag())];
         }
         
         // separate write access
         void set_element(std::size_t i,std::size_t j, value_type t){
+                SIZE_CHECK(i < size1());
+                SIZE_CHECK(j < size2());
                 SIZE_CHECK(orientation::non_zero(i,j));
-                m_data [orientation::element(i,j,size1())] = t;
+                m_data [orientation::element(i,j,size1(),packed_tag())] = t;
         }
         
         bool non_zero(std::size_t i,std::size_t j)const{
+                SIZE_CHECK(i < size1());
+                SIZE_CHECK(j < size2());
                 return orientation::non_zero(i,j);
         }
         
-        // Assignment
-        template<class E>
-        triangular_matrix& assign(matrix_expression<E> const& e) {
-                kernels::assign(*this,e);
-                return *this;
-        }
-        template<class E>
-        triangular_matrix& plus_assign(matrix_expression<E> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                kernels::assign<scalar_plus_assign> (*this, e);
-                return *this;
-        }
-        template<class E>
-        triangular_matrix& minus_assign(matrix_expression<E> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                kernels::assign<scalar_minus_assign> (*this, e);
-                return *this;
-        }
-        
-        template<class E>
-        triangular_matrix& multiply_assign(matrix_expression<E> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                kernels::assign<scalar_multiply_assign> (*this, e);
-                return *this;
-        }
-        template<class E>
-        triangular_matrix& divide_assign(matrix_expression<E> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                kernels::assign<scalar_divide_assign> (*this, e);
-                return *this;
-        }
-        
         /*! @note "pass by value" the key idea to enable move semantics */
         triangular_matrix& operator = (triangular_matrix m) {
                 swap(m);
@@ -175,7 +182,7 @@ public:
         triangular_matrix& operator = (const matrix_container<C>& m) {
                 SIZE_CHECK(m().size1()==m().size2());
                 resize(m().size1());
-                assign(m);
+                assign(*this, m);
                 return *this;
         }
         template<class E>
@@ -184,74 +191,6 @@ public:
                 swap(temporary);
                 return *this;
         }
-        template<class E>
-        triangular_matrix& operator += (matrix_expression<E> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                self_type temporary(*this + e);
-                swap(temporary);
-                return *this;
-        }
-        template<class C>          // Container assignment without temporary
-        triangular_matrix& operator += (const matrix_container<C>& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                return plus_assign(e);
-        }
-        
-        template<class E>
-        triangular_matrix& operator -= (matrix_expression<E> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                self_type temporary(*this - e);
-                swap(temporary);
-                return *this;
-        }
-        template<class C>          // Container assignment without temporary
-        triangular_matrix& operator -= (matrix_container<C> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                return minus_assign(e);
-        }
-        
-        template<class E>
-        triangular_matrix& operator *= (matrix_expression<E> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                self_type temporary(*this * e);
-                swap(temporary);
-                return *this;
-        }
-        template<class C>          // Container assignment without temporary
-        triangular_matrix& operator *= (const matrix_container<C>& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                return multiply_assign(e);
-        }
-        
-        template<class E>
-        triangular_matrix& operator /= (matrix_expression<E> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                self_type temporary(*this / e);
-                swap(temporary);
-                return *this;
-        }
-        template<class C>          // Container assignment without temporary
-        triangular_matrix& operator /= (matrix_container<C> const& e) {
-                SIZE_CHECK(size1() == e().size1());
-                SIZE_CHECK(size2() == e().size2());
-                return divide_assign(e);
-        }
-        
-        triangular_matrix& operator *= (scalar_type t) {
-                kernels::assign<scalar_multiply_assign> (*this, t);
-                return *this;
-        }
-        triangular_matrix& operator /= (scalar_type t) {
-                kernels::assign<scalar_divide_assign> (*this, t);
-                return *this;
-        }
 
         // Swapping
         void swap(triangular_matrix& m) {
@@ -456,7 +395,7 @@ public:
         
         typedef typename boost::mpl::if_<
                 boost::is_same<Orientation,row_major>,
-                dense_storage_iterator<value_type>,
+                dense_storage_iterator<value_type,packed_random_access_iterator_tag>,
                 typename boost::mpl::if_c<
                         TriangularType::is_upper,
                         major1_iterator<value_type>,
@@ -470,12 +409,12 @@ public:
                         major2_iterator<value_type>,
                         major1_iterator<value_type>
                 >::type,
-                dense_storage_iterator<value_type>
+                dense_storage_iterator<value_type,packed_random_access_iterator_tag>
         >::type column_iterator;
         
         typedef typename boost::mpl::if_<
                 boost::is_same<Orientation,row_major>,
-                dense_storage_iterator<value_type const>,
+                dense_storage_iterator<value_type const,packed_random_access_iterator_tag>,
                 typename boost::mpl::if_c<
                         TriangularType::is_upper,
                         major1_iterator<value_type const>,
@@ -489,7 +428,7 @@ public:
                         major2_iterator<value_type const>,
                         major1_iterator<value_type const>
                 >::type,
-                dense_storage_iterator<value_type const>
+                dense_storage_iterator<value_type const,packed_random_access_iterator_tag>
         >::type const_column_iterator;
         
 public:
@@ -497,66 +436,66 @@ public:
         const_row_iterator row_begin(index_type i) const {
                 std::size_t index = TriangularType::is_upper?i:0;
                 return const_row_iterator(
-                        &m_data[orientation::element(i,index,size1())]
+                        storage()+orientation::element(i,index,size1(),packed_tag())
                         ,index
-                        ,orientation::stride2(size1(),size2())//1 if row_major, size2() otherwise
+                        ,orientation::orientation::stride2(size1(),size2())//1 if row_major, size2() otherwise
                 );
         }
         const_row_iterator row_end(index_type i) const {
                 std::size_t index = TriangularType::is_upper?size2():i+1;
                 return const_row_iterator(
-                        &m_data[orientation::element(i,index,size1())]
+                        storage() + orientation::element(i, index, size1(),packed_tag())
                         ,index
-                        ,orientation::stride2(size1(),size2())//1 if row_major, size2() otherwise
+                        ,orientation::orientation::stride2(size1(),size2())//1 if row_major, size2() otherwise
                 );
         }
         row_iterator row_begin(index_type i){
                 std::size_t index = TriangularType::is_upper?i:0;
                 return row_iterator(
-                        &m_data[orientation::element(i,index,size1())]
+                        storage() + orientation::element(i, index, size1(),packed_tag())
                         ,index
-                        ,orientation::stride2(size1(),size2())//1 if row_major, size2() otherwise
+                        ,orientation::orientation::stride2(size1(),size2())//1 if row_major, size2() otherwise
                 );
         }
         row_iterator row_end(index_type i){
                 std::size_t index = TriangularType::is_upper?size2():i+1;
                 return row_iterator(
-                        &m_data[orientation::element(i,index,size1())]
+                        storage() + orientation::element(i, index, size1(),packed_tag())
                         ,index
-                        ,orientation::stride2(size1(),size2())//1 if row_major, size2() otherwise
+                        ,orientation::orientation::stride2(size1(),size2())//1 if row_major, size2() otherwise
                 );
         }
         
         const_column_iterator column_begin(index_type i) const {
                 std::size_t index = TriangularType::is_upper?0:i;
                 return const_column_iterator(
-                        &m_data[orientation::element(index,i,size1())]
+                        storage() + orientation::element(index, i, size1(),packed_tag())
                         ,index
-                        ,orientation::stride1(size1(),size2())//size1() if row_major, 1 otherwise
+                        ,orientation::orientation::stride1(size1(),size2())//size1() if row_major, 1 otherwise
                 );
         }
         const_column_iterator column_end(index_type i) const {
                 std::size_t index = TriangularType::is_upper?i+1:size2();
                 return const_column_iterator(
-                        &m_data[orientation::element(index,i,size1())]
+                        storage() + orientation::element(index, i, size1(),packed_tag())
                         ,index
-                        ,orientation::stride1(size1(),size2())//size1() if row_major, 1 otherwise
+                        ,orientation::orientation::stride1(size1(),size2())//size1() if row_major, 1 otherwise
                 );
         }
         column_iterator column_begin(index_type i){
                 std::size_t index = TriangularType::is_upper?0:i;
                 return column_iterator(
-                        &m_data[orientation::element(index,i,size1())]
+                        storage() + orientation::element(index, i, size1(),packed_tag())
                         ,index
-                        ,orientation::stride1(size1(),size2())//size1() if row_major, 1 otherwise
+                        ,orientation::orientation::stride1(size1(),size2())//size1() if row_major, 1 otherwise
                 );
         }
         column_iterator column_end(index_type i){
                 std::size_t index = TriangularType::is_upper?i+1:size2();
                 return column_iterator(
-                        &m_data[orientation::element(index,i,size1())]
+                        storage() + orientation::element(index, i, size1(),packed_tag())
                         ,index
-                        ,orientation::stride1(size1(),size2())//size1() if row_major, 1 otherwise
+                        ,orientation::orientation::stride1(size1(),size2())//size1() if row_major, 1 otherwise
                 );
         }
 
@@ -566,7 +505,17 @@ public:
         array_type m_data;
 };
 
+template<class T, class Orientation, class TriangularType>
+struct const_expression<triangular_matrix<T,Orientation, TriangularType> >{
+        typedef triangular_matrix<T,Orientation, TriangularType> const type;
+};
+template<class T, class Orientation, class TriangularType>
+struct const_expression<triangular_matrix<T,Orientation, TriangularType> const>{
+        typedef triangular_matrix<T,Orientation, TriangularType> const type;
+};
+
 }
 }
 
 #endif
+
