@@ -1,14 +1,149 @@
+// [[Rcpp::depends(BH)]]
+/*!
+ * \brief       expression templates for vector valued math
+ * 
+ * \author      O. Krause
+ * \date        2013
+ *
+ *
+ * \par Copyright 1995-2015 Shark Development Team
+ * 
+ * <BR><HR>
+ * This file is part of Shark.
+ * <http://image.diku.dk/shark/>
+ * 
+ * Shark is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published 
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * Shark is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with Shark.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ */
 #ifndef SHARK_LINALG_BLAS_VECTOR_EXPRESSION_HPP
 #define SHARK_LINALG_BLAS_VECTOR_EXPRESSION_HPP
 
-#include <boost/type_traits/is_convertible.hpp> 
-#include <boost/utility/enable_if.hpp> 
 #include "vector_proxy.hpp"
 #include "kernels/dot.hpp"
+#include <boost/utility/enable_if.hpp>
 
 namespace shark {
 namespace blas {
 	
+///\brief Implements multiplications of a vector by a scalar
+template<class E>
+class vector_scalar_multiply:
+	public vector_expression<vector_scalar_multiply <E> > {
+	typedef vector_scalar_multiply<E> self_type;
+public:
+	typedef typename E::const_closure_type expression_closure_type;
+	typedef typename E::size_type size_type;
+	typedef typename E::difference_type difference_type;
+	typedef typename E::value_type value_type;
+	typedef typename E::scalar_type scalar_type;
+	typedef value_type const_reference;
+	typedef value_type reference;
+	typedef value_type const * const_pointer;
+	typedef value_type const*  pointer;
+
+	typedef typename E::index_type index_type;
+	typedef typename E::const_index_pointer const_index_pointer;
+	typedef typename index_pointer<E>::type index_pointer;
+
+	typedef self_type const_closure_type;
+	typedef self_type closure_type;
+	typedef unknown_storage_tag storage_category;
+	typedef typename E::evaluation_category evaluation_category;
+
+	// Construction and destruction
+	// May be used as mutable expression.
+	vector_scalar_multiply(expression_closure_type const& e, scalar_type scalar):
+		m_expression(e), m_scalar(scalar) {}
+
+	// Accessors
+	size_type size() const {
+		return m_expression.size();
+	}
+
+	// Expression accessors
+	expression_closure_type const &expression() const {
+		return m_expression;
+	}
+
+public:
+	// Element access
+	const_reference operator()(index_type i) const {
+		return m_scalar * m_expression(i);
+	}
+
+	const_reference operator[](index_type i) const {
+		return m_scalar * m_expression(i);
+	}
+	
+	
+	//computation kernels
+	template<class VecX>
+	void assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		m_expression.assign_to(x,alpha*m_scalar);
+	}
+	template<class VecX>
+	void plus_assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		m_expression.plus_assign_to(x,alpha*m_scalar);
+	}
+	
+	template<class VecX>
+	void minus_assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		m_expression.minus_assign_to(x,alpha*m_scalar);
+	}
+
+	
+	//iterators
+	typedef transform_iterator<typename E::const_iterator,scalar_multiply1<value_type, scalar_type> > const_iterator;
+	typedef const_iterator iterator;
+	
+	const_iterator begin() const {
+		return const_iterator(m_expression.begin(),scalar_multiply1<value_type, scalar_type>(m_scalar));
+	}
+	const_iterator end() const {
+		return const_iterator(m_expression.end(),scalar_multiply1<value_type, scalar_type>(m_scalar));
+	}
+private:
+	expression_closure_type m_expression;
+	scalar_type m_scalar;
+};
+
+
+template<class T, class E>
+typename boost::enable_if<
+	std::is_convertible<T, typename E::scalar_type >,
+        vector_scalar_multiply<E>
+>::type
+operator* (vector_expression<E> const& e, T scalar){
+	typedef typename E::scalar_type scalar_type;
+	return vector_scalar_multiply<E>(e(), scalar_type(scalar));
+}
+template<class T, class E>
+typename boost::enable_if<
+	std::is_convertible<T, typename E::scalar_type >,
+        vector_scalar_multiply<E>
+>::type
+operator* (T scalar, vector_expression<E> const& e){
+	typedef typename E::scalar_type scalar_type;
+	return vector_scalar_multiply<E>(e(), scalar_type(scalar));//explicit cast prevents warning, alternative would be to template vector_scalar_multiply on T as well
+}
+
+template<class E>
+vector_scalar_multiply<E> operator-(vector_expression<E> const& e){
+	typedef typename E::scalar_type scalar_type;
+	return vector_scalar_multiply<E>(e(), scalar_type(-1));//explicit cast prevents warning, alternative would be to template vector_scalar_multiply on T as well
+}
+	
 /// \brief Vector expression representing a constant valued vector.
 template<class T>
 class scalar_vector:public vector_expression<scalar_vector<T> > {
@@ -20,17 +155,19 @@ public:
 	typedef T value_type;
 	typedef T scalar_type;
 	typedef const T& const_reference;
-	typedef T& reference;
-	typedef T* pointer;
-	typedef const T *const_pointer;
+	typedef const_reference reference;
+	typedef T const* const_pointer;
+	typedef const_pointer pointer;
+	
 
 	typedef std::size_t index_type;
 	typedef index_type const* const_index_pointer;
 	typedef index_type index_pointer;
 
-	typedef const vector_reference<const self_type> const_closure_type;
-	typedef vector_reference<self_type> closure_type;
+	typedef self_type const_closure_type;
+	typedef self_type closure_type;
 	typedef unknown_storage_tag storage_category;
+	typedef elementwise_tag evaluation_category;
 
 	// Construction and destruction
 	scalar_vector()
@@ -45,11 +182,6 @@ public:
 		return m_size;
 	}
 
-	// Resizing
-	void resize(size_type size, bool /*preserve*/ = true) {
-		m_size = size;
-	}
-
 	// Element access
 	const_reference operator()(index_type /*i*/) const {
 		return m_value;
@@ -80,7 +212,7 @@ private:
 ///@param scalar the value which is repeated
 ///@param elements the size of the resulting vector
 template<class T>
-typename boost::enable_if<boost::is_arithmetic<T>, scalar_vector<T> >::type
+typename boost::enable_if<std::is_arithmetic<T>, scalar_vector<T> >::type
 repeat(T scalar, std::size_t elements){
 	return scalar_vector<T>(elements,scalar);
 }
@@ -95,8 +227,6 @@ template<class E, class F>
 class vector_unary:
 	public vector_expression<vector_unary<E, F> > {
 	typedef vector_unary<E, F> self_type;
-	typedef E const expression_type;
-
 public:
 	typedef F functor_type;
 	typedef typename E::const_closure_type expression_closure_type;
@@ -113,9 +243,10 @@ public:
 	typedef typename E::const_index_pointer const_index_pointer;
 	typedef typename index_pointer<E>::type index_pointer;
 
-	typedef self_type const const_closure_type;
+	typedef self_type const_closure_type;
 	typedef self_type closure_type;
 	typedef unknown_storage_tag storage_category;
+	typedef typename E::evaluation_category evaluation_category;
 
 	// Construction and destruction
 	// May be used as mutable expression.
@@ -131,6 +262,33 @@ public:
 	expression_closure_type const &expression() const {
 		return m_expression;
 	}
+	
+	//computation kernels
+	template<class VecX>
+	void assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		//compute this by first assigning the result of the argument and then applying
+		//the function to every element
+		assign(x,m_expression);
+		typename VecX::iterator end=x().end();
+		for(typename VecX::iterator pos =x().begin(); pos != end; ++pos){
+			*pos= alpha * m_functor(*pos);
+		}
+	}
+	template<class VecX>
+	void plus_assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		//First assign result of this expression to a temporary and then perform plus_assignment to x
+		typename vector_temporary<self_type>::type temporary(size());
+		assign_to(temporary,alpha);
+		plus_assign_to(x,temporary);
+	}
+	
+	template<class VecX>
+	void minus_assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		//First assign result of this expression to a temporary and then perform minus_assignment to x
+		typename vector_temporary<self_type>::type temporary(size());
+		assign_to(temporary,alpha);
+		minus_assign_to(x,temporary);
+	}
 
 public:
 	// Element access
@@ -142,11 +300,6 @@ public:
 		return m_functor(m_expression[i]);
 	}
 
-	// Closure comparison
-	bool same_closure(vector_unary const &vu) const {
-		return expression().same_closure(vu.expression());
-	}
-
 	typedef transform_iterator<typename E::const_iterator,functor_type> const_iterator;
 	typedef const_iterator iterator;
 
@@ -171,7 +324,6 @@ name(vector_expression<E> const& e){\
 	typedef F<typename E::value_type> functor_type;\
 	return vector_unary<E, functor_type>(e, functor_type());\
 }
-SHARK_UNARY_VECTOR_TRANSFORMATION(operator-, scalar_negate)
 SHARK_UNARY_VECTOR_TRANSFORMATION(abs, scalar_abs)
 SHARK_UNARY_VECTOR_TRANSFORMATION(log, scalar_log)
 SHARK_UNARY_VECTOR_TRANSFORMATION(exp, scalar_exp)
@@ -192,16 +344,13 @@ SHARK_UNARY_VECTOR_TRANSFORMATION(elem_inv, scalar_inverse)
 #define SHARK_VECTOR_SCALAR_TRANSFORMATION(name, F)\
 template<class T, class E> \
 typename boost::enable_if< \
-	boost::is_convertible<T, typename E::value_type >,\
+	std::is_convertible<T, typename E::value_type >,\
         vector_unary<E,F<typename E::value_type,T> > \
 >::type \
 name (vector_expression<E> const& e, T scalar){ \
 	typedef F<typename E::value_type,T> functor_type; \
 	return vector_unary<E, functor_type>(e, functor_type(scalar)); \
 }
-SHARK_VECTOR_SCALAR_TRANSFORMATION(operator+, scalar_add)
-SHARK_VECTOR_SCALAR_TRANSFORMATION(operator-, scalar_subtract2)
-SHARK_VECTOR_SCALAR_TRANSFORMATION(operator*, scalar_multiply2)
 SHARK_VECTOR_SCALAR_TRANSFORMATION(operator/, scalar_divide)
 SHARK_VECTOR_SCALAR_TRANSFORMATION(operator<, scalar_less_than)
 SHARK_VECTOR_SCALAR_TRANSFORMATION(operator<=, scalar_less_equal_than)
@@ -218,20 +367,187 @@ SHARK_VECTOR_SCALAR_TRANSFORMATION(pow, scalar_pow)
 #define SHARK_VECTOR_SCALAR_TRANSFORMATION_2(name, F)\
 template<class T, class E> \
 typename boost::enable_if< \
-	boost::is_convertible<T, typename E::value_type >,\
+	std::is_convertible<T, typename E::value_type >,\
         vector_unary<E,F<typename E::value_type,T> > \
 >::type \
 name (T scalar, vector_expression<E> const& e){ \
 	typedef F<typename E::value_type,T> functor_type; \
 	return vector_unary<E, functor_type>(e, functor_type(scalar)); \
 }
-SHARK_VECTOR_SCALAR_TRANSFORMATION_2(operator+, scalar_add)
-SHARK_VECTOR_SCALAR_TRANSFORMATION_2(operator-, scalar_subtract1)
-SHARK_VECTOR_SCALAR_TRANSFORMATION_2(operator*, scalar_multiply1)
 SHARK_VECTOR_SCALAR_TRANSFORMATION_2(min, scalar_min)
 SHARK_VECTOR_SCALAR_TRANSFORMATION_2(max, scalar_max)
 #undef SHARK_VECTOR_SCALAR_TRANSFORMATION_2
 
+template<class E1, class E2>
+class vector_addition: public vector_expression<vector_addition<E1,E2> > {
+private:
+	typedef scalar_binary_plus<
+		typename E1::value_type,
+		typename E2::value_type
+	> functor_type;
+public:
+	typedef std::size_t size_type;
+	typedef std::ptrdiff_t difference_type;
+	typedef typename functor_type::result_type value_type;
+	typedef value_type scalar_type;
+	typedef value_type const_reference;
+	typedef value_type reference;
+	typedef value_type const * const_pointer;
+	typedef value_type const*  pointer;
+
+	typedef typename E1::index_type index_type;
+	typedef typename E1::const_index_pointer const_index_pointer;
+	typedef typename index_pointer<E1>::type index_pointer;
+
+	typedef typename E1::const_closure_type expression_closure1_type;
+	typedef typename E2::const_closure_type expression_closure2_type;
+	
+	typedef vector_addition const_closure_type;
+	typedef vector_addition closure_type;
+	typedef unknown_storage_tag storage_category;
+	typedef typename evaluation_restrict_traits<E1,E2>::type evaluation_category;
+
+	// Construction and destruction
+	explicit vector_addition (
+		expression_closure1_type e1, 
+		expression_closure2_type e2
+	):m_expression1(e1),m_expression2(e2){
+		SIZE_CHECK(e1.size() == e2.size());
+	}
+
+	// Accessors
+	size_type size() const {
+		return m_expression1.size();
+	}
+
+	// Expression accessors
+	expression_closure1_type const& expression1() const {
+		return m_expression1;
+	}
+	expression_closure2_type const& expression2() const {
+		return m_expression2;
+	}
+
+	// Element access
+	const_reference operator() (index_type i) const {
+		SIZE_CHECK(i < size());
+		return m_expression1(i) + m_expression2(i);
+	}
+
+	const_reference operator[] (index_type i) const {
+		SIZE_CHECK(i < size());
+		return m_expression1(i) + m_expression2(i);
+	}
+	
+	//computation kernels
+	template<class VecX>
+	void assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		assign(x,alpha*m_expression1);
+		plus_assign(x,alpha*m_expression2);
+	}
+	template<class VecX>
+	void plus_assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		plus_assign(x,alpha*m_expression1);
+		plus_assign(x,alpha*m_expression2);
+	}
+	
+	template<class VecX>
+	void minus_assign_to(vector_expression<VecX>& x, scalar_type alpha = scalar_type(1) )const{
+		minus_assign(x,alpha*m_expression1);
+		minus_assign(x,alpha*m_expression2);
+	}
+
+	// Iterator types
+	typedef binary_transform_iterator<
+		typename E1::const_iterator,
+		typename E2::const_iterator,
+		functor_type
+	> const_iterator;
+	typedef const_iterator iterator;
+
+	const_iterator begin () const {
+		return const_iterator(functor_type(),
+			m_expression1.begin(),m_expression1.end(),
+			m_expression2.begin(),m_expression2.end()
+		);
+	}
+	const_iterator end() const {
+		return const_iterator(functor_type(),
+			m_expression1.end(),m_expression1.end(),
+			m_expression2.end(),m_expression2.end()
+		);
+	}
+
+private:
+	expression_closure1_type m_expression1;
+	expression_closure2_type m_expression2;
+};
+
+///\brief Adds two vectors
+template<class E1, class E2>
+vector_addition<E1, E2 > operator+ (
+	vector_expression<E1> const& e1,
+	vector_expression<E2> const& e2
+){
+	return vector_addition<E1, E2>(e1(),e2());
+}
+///\brief Subtracts two vectors
+template<class E1, class E2>
+vector_addition<E1, vector_scalar_multiply<E2> > operator- (
+	vector_expression<E1> const& e1,
+	vector_expression<E2> const& e2
+){
+	return vector_addition<E1, vector_scalar_multiply<E2> >(e1(),-e2());
+}
+
+///\brief Adds a vector plus a scalr which is interpreted as a constant vector
+template<class E, class T>
+typename boost::enable_if<
+	std::is_convertible<T, typename E::value_type>, 
+	vector_addition<E, scalar_vector<T> >
+>::type operator+ (
+	vector_expression<E> const& e,
+	T t
+){
+	return e + scalar_vector<T>(e().size(),t);
+}
+
+///\brief Adds a vector plus a scalar which is interpreted as a constant vector
+template<class T, class E>
+typename boost::enable_if<
+	std::is_convertible<T, typename E::value_type>,
+	vector_addition<E, scalar_vector<T> >
+>::type operator+ (
+	T t,
+	vector_expression<E> const& e
+){
+	return e + scalar_vector<T>(e().size(),t);
+}
+
+///\brief Subtracts a scalar which is interpreted as a constant vector from a vector.
+template<class E, class T>
+typename boost::enable_if<
+	std::is_convertible<T, typename E::value_type> ,
+	vector_addition<E, vector_scalar_multiply<scalar_vector<T> > >
+>::type operator- (
+	vector_expression<E> const& e,
+	T t
+){
+	return e - scalar_vector<T>(e().size(),t);
+}
+
+///\brief Subtracts a vector from a scalar which is interpreted as a constant vector
+template<class E, class T>
+typename boost::enable_if<
+	std::is_convertible<T, typename E::value_type>,
+	vector_addition<scalar_vector<T>, vector_scalar_multiply<E> >
+>::type operator- (
+	T t,
+	vector_expression<E> const& e
+){
+	return scalar_vector<T>(e().size(),t) - e;
+}
+
 
 template<class E1, class E2, class F>
 class vector_binary:
@@ -257,9 +573,10 @@ public:
 	typedef typename E1::const_closure_type expression_closure1_type;
 	typedef typename E2::const_closure_type expression_closure2_type;
 	
-	typedef self_type const const_closure_type;
+	typedef self_type const_closure_type;
 	typedef self_type closure_type;
 	typedef unknown_storage_tag storage_category;
+	typedef typename evaluation_restrict_traits<E1,E2>::type evaluation_category;
 
 	// Construction and destruction
 	explicit vector_binary (
@@ -294,18 +611,14 @@ public:
 		return m_functor(m_expression1(i),m_expression2(i));
 	}
 
-	// Closure comparison
-	bool same_closure (vector_binary const& vu) const {
-		return expression1 ().same_closure (vu.expression1())
-		&& expression2 ().same_closure (vu.expression2());
-	}
-
 	// Iterator types
 	
 	// Iterator enhances the iterator of the referenced expressions
 	// with the unary functor.
 	typedef binary_transform_iterator<
-		typename E1::const_iterator,typename E2::const_iterator,functor_type
+		typename E1::const_iterator,
+		typename E2::const_iterator,
+		functor_type
 	> const_iterator;
 	typedef const_iterator iterator;
 
@@ -335,8 +648,6 @@ name(vector_expression<E1> const& e1, vector_expression<E2> const& e2){\
 	typedef F<typename E1::value_type, typename E2::value_type> functor_type;\
 	return vector_binary<E1, E2, functor_type>(e1(),e2(), functor_type());\
 }
-SHARK_BINARY_VECTOR_EXPRESSION(operator+, scalar_binary_plus)
-SHARK_BINARY_VECTOR_EXPRESSION(operator-, scalar_binary_minus)
 SHARK_BINARY_VECTOR_EXPRESSION(operator*, scalar_binary_multiply)
 SHARK_BINARY_VECTOR_EXPRESSION(element_prod, scalar_binary_multiply)
 SHARK_BINARY_VECTOR_EXPRESSION(operator/, scalar_binary_divide)
@@ -352,10 +663,9 @@ vector_binary<E1, E2,
 safe_div(
 	vector_expression<E1> const& e1, 
 	vector_expression<E2> const& e2, 
-	typename promote_traits<
-		typename E1::value_type, 
-		typename E2::value_type
-	>::promote_type defaultValue
+	decltype(
+		typename E1::value_type() * typename E2::value_type()
+	) defaultValue
 ){
 	typedef scalar_binary_safe_divide<typename E1::value_type, typename E2::value_type> functor_type;
 	return vector_binary<E1, E2, functor_type>(e1(),e2(), functor_type(defaultValue));
@@ -415,7 +725,7 @@ template<class E>
 typename E::value_type
 soft_max(const vector_expression<E> &e) {
 	typename E::value_type maximum = max(e);
-	return std::log(sum(exp(e-blas::repeat(maximum,e().size()))))+maximum;
+	return std::log(sum(exp(e - maximum))) + maximum;
 }
 
 
@@ -425,14 +735,14 @@ soft_max(const vector_expression<E> &e) {
 template<class E>
 typename real_traits<typename E::value_type >::type
 norm_1(const vector_expression<E> &e) {
-	return sum(abs(e));
+	return sum(abs(eval_block(e)));
 }
 
 /// \brief norm_2 v = sum_i |v_i|^2
 template<class E>
 typename real_traits<typename E::value_type >::type
 norm_sqr(const vector_expression<E> &e) {
-	return sum(abs_sqr(e));
+	return sum(abs_sqr(eval_block(e)));
 }
 
 /// \brief norm_2 v = sqrt (sum_i |v_i|^2 )
@@ -447,31 +757,29 @@ norm_2(const vector_expression<E> &e) {
 template<class E>
 typename real_traits<typename E::value_type >::type
 norm_inf(vector_expression<E> const &e){
-	return max(abs(e));
+	return max(abs(eval_block(e)));
 }
 
 /// \brief index_norm_inf v = arg max_i |v_i|
 template<class E>
 std::size_t index_norm_inf(vector_expression<E> const &e){
-	return arg_max(abs(e));
+	return arg_max(abs(eval_block(e)));
 }
 
 // inner_prod (v1, v2) = sum_i v1_i * v2_i
 template<class E1, class E2>
-typename promote_traits<
-	typename E1::value_type,
-	typename E2::value_type
->::promote_type
+decltype(
+	typename E1::value_type() * typename E2::value_type()
+)
 inner_prod(
 	vector_expression<E1> const& e1,
 	vector_expression<E2> const& e2
 ) {
-	typedef typename promote_traits<
-		typename E1::value_type,
-		typename E2::value_type
-	>::promote_type value_type;
+	typedef decltype(
+		typename E1::value_type() * typename E2::value_type()
+	) value_type;
 	value_type result = value_type();
-	kernels::dot(e1,e2,result);
+	kernels::dot(eval_block(e1),eval_block(e2),result);
 	return result;
 }
 
@@ -480,3 +788,4 @@ inner_prod(
 }
 
 #endif
+
